@@ -1,16 +1,15 @@
 use std::future::Future;
 use std::pin::Pin;
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
-use std::sync::Arc;
+use std::sync::{Arc, Mutex, RwLock};
 use std::task::{Context, Poll, Waker};
 
 use async_task::{Runnable, Task};
-use concurrent_queue::ConcurrentQueue;
 use futures_util::future::poll_fn;
-use parking_lot::{Mutex, RwLock};
-
 use rand::Rng;
 
+use crate::queue::Queue;
+
 /// A multi-threaded executor.
 #[derive(Debug)]
 pub struct Executor {
@@ -27,7 +26,7 @@ impl Executor {
     pub fn new() -> Executor {
         Executor {
             global: Arc::new(Global {
-                queue: ConcurrentQueue::unbounded(),
+                queue: Queue::new(),
                 notified: AtomicBool::new(false),
                 shards: RwLock::new(Vec::new()),
                 sleepers: Mutex::new(Sleepers {
@@ -44,28 +43,28 @@ impl Executor {
         future: impl Future<Output = T> + Send + 'static,
     ) -> Task<T> {
         let global = self.global.clone();
-
         let schedule = move |runnable| {
-            global.queue.push(runnable).unwrap();
+            global.queue.push(runnable);
             global.notify();
         };
 
         let (runnable, task) = async_task::spawn(future, schedule);
         runnable.schedule();
-
         task
     }
 
     pub fn ticker(&self) -> Ticker {
         let ticker = Ticker {
             global: self.global.clone(),
-            shard: Arc::new(ConcurrentQueue::bounded(512)),
+            shard: Arc::new(Queue::with_capacity(512)),
             sleeping: AtomicUsize::new(0),
             ticks: AtomicUsize::new(0),
         };
-
-        self.global.shards.write().push(ticker.shard.clone());
-
+        self.global
+            .shards
+            .write()
+            .unwrap()
+            .push(ticker.shard.clone());
         ticker
     }
 }
@@ -73,13 +72,13 @@ impl Executor {
 #[derive(Debug)]
 struct Global {
     /// The global queue.
-    queue: ConcurrentQueue<Runnable>,
+    queue: Queue<Runnable>,
 
     /// Set to `true` when a sleeping ticker is notified or no tickers are sleeping.
     notified: AtomicBool,
 
     /// Shards of the global queue created by tickers.
-    shards: RwLock<Vec<Arc<ConcurrentQueue<Runnable>>>>,
+    shards: RwLock<Vec<Arc<Queue<Runnable>>>>,
 
     /// A list of sleeping tickers.
     sleepers: Mutex<Sleepers>,
@@ -162,7 +161,7 @@ impl Global {
             .compare_exchange(false, true, Ordering::SeqCst, Ordering::SeqCst)
             .is_ok()
         {
-            let waker = self.sleepers.lock().notify();
+            let waker = self.sleepers.lock().unwrap().notify();
             if let Some(waker) = waker {
                 waker.wake();
             }
@@ -177,7 +176,7 @@ pub struct Ticker {
     global: Arc<Global>,
 
     /// A shard of the global queue.
-    shard: Arc<ConcurrentQueue<Runnable>>,
+    shard: Arc<Queue<Runnable>>,
 
     /// Set to `sleeper's id` when in sleeping state.
     ///
@@ -196,7 +195,7 @@ impl Ticker {
     ///
     /// Returns `false` if the ticker was already sleeping and unnotified.
     fn sleep(&self, waker: &Waker) -> bool {
-        let mut sleepers = self.global.sleepers.lock();
+        let mut sleepers = self.global.sleepers.lock().unwrap();
 
         match self.sleeping.load(Ordering::SeqCst) {
             // Move to sleeping state.
@@ -224,7 +223,7 @@ impl Ticker {
             return;
         }
 
-        let mut sleepers = self.global.sleepers.lock();
+        let mut sleepers = self.global.sleepers.lock().unwrap();
         sleepers.remove(id);
 
         self.global
@@ -275,17 +274,17 @@ impl Ticker {
 
     /// Finds the next task to run.
     fn search(&self) -> Option<Runnable> {
-        if let Ok(r) = self.shard.pop() {
+        if let Some(r) = self.shard.pop() {
             return Some(r);
         }
 
         // Try stealing from the global queue.
-        if let Ok(r) = self.global.queue.pop() {
+        if let Some(r) = self.global.queue.pop() {
             return Some(r);
         }
 
         // Try stealing from other shards.
-        let shards = self.global.shards.read();
+        let shards = self.global.shards.read().unwrap();
 
         // Pick a random starting point in the iterator list and rotate the list.
         let n = shards.len();
@@ -300,7 +299,7 @@ impl Ticker {
         // Try stealing from each shard in the list.
         for shard in iter {
             steal(shard, &self.shard);
-            if let Ok(r) = self.shard.pop() {
+            if let Some(r) = self.shard.pop() {
                 return Some(r);
             }
         }
@@ -310,20 +309,16 @@ impl Ticker {
 }
 
 /// Steals some items from one queue into another.
-fn steal<T>(src: &ConcurrentQueue<T>, dest: &ConcurrentQueue<T>) {
+fn steal<T>(src: &Queue<T>, dest: &Queue<T>) {
     // Half of `src`'s length rounded up.
     let mut count = (src.len() + 1) / 2;
 
     if count > 0 {
         // Don't steal more than fits into the queue.
-        if let Some(cap) = dest.capacity() {
-            count = count.min(cap - dest.len());
-        }
-
-        // Steal tasks.
+        count = count.min(dest.capacity() - dest.len());
         for _ in 0..count {
-            if let Ok(t) = src.pop() {
-                assert!(dest.push(t).is_ok());
+            if let Some(t) = src.pop() {
+                dest.push(t);
             } else {
                 break;
             }
